/*
-------------------------------------------------------------------------------
-------------------------------------------------------------------------------
*************************** 602_Programm_Gleitkomma ***************************
-------------------------------------------------------------------------------
******************************* main.c ** V1.1 ********************************
-------------------------------------------------------------------------------
-------------------------------------------------------------------------------
 Darstellung einer Gleitkomma-Zahl auf dem LCD
 Zusaetzliche Header..: keine
 Module.(*.c und *.h).: application, lcd_2wire, num_conversion
 Benoetigte Hardware..: Testboard, 20x4-LCD und 2-Draht-LCD-Interface
 Version..............: 1.1
 Compiler.............: CodeVisionAVR
 Chip.................: ATmega88
 Datum................: Juni 2009
 Autor................: Alwin Lenck (ALE23) basierend auf dem Projekt 
                        2-Draht-LCD von Udo Juerss
-------------------------------------------------------------------------------
Darstellung einer Gleitkomma-Zahl auf dem 20x4-LCD in ihren "Bestandteilen" 

Anschluesse des 20x4-LCD:
CLOCK................: PB1 (kann geaendert werden in "application.h")
DATA.................: PB2 (kann geaendert werden in "application.h")

-------------------------------------------------------------------------------
 Verwendete Typ-Synonyme (werden in typedefs.h definiert)
-------------------------------------------------------------------------------
 U08    unsigned char
 U32    unsigned long int
 fpU08  unsigned char __flash *...        // Pointer auf unsigned char im FLASH
*/

//-----------------------------------------------------------------------------
// Header-Dateien einfuegen:
//-----------------------------------------------------------------------------
#include "application.h"                  // Initialisierung der Anwendung
#include "lcd_2wire.h"                    // Initialisierung des 2Draht-LCDs
#include "num_conversion.h"               // Initialisierung der Konvertierung

//-----------------------------------------------------------------------------
// Globale Variablen:
//-----------------------------------------------------------------------------
fpU08 LCD_STR[4] =                        // Vordef. Display-Text im FLASH
{
  ("String:   +12345.678"),               // Gleitkomma-Zahl als String
  ("Vor:         xxxxxxx"),               // Wert vor dem Dezimalpunkt
  ("Nach:        zzzzzzz"),               // Wert nach dem Dezimalpunkt
  ("Float:     fffffffff")                // Darstellung der Gleitkomma-Zahl
};                                        // max. 9 Zeichen: Vorzeichen,
                                          // 7 Ziffern und Dezimalpunkt

// Zu wandelnder Zahlenwert - in der Mantisse max. darstellbarer Wert 16777215
float float_wert  =   -1677.7215;         // Zu wandelnde Gleitkomma-Zahl
U08 char_wert[13] ="  -1677.7215";        // Wiederholung als String

U08   n, i;                               // Zaehler
U32   int_wert;                           // Integer-Wert, Wert vor dem Punkt
float dec_wert;                           // Ziffernfolge nach dem Dezimalpunkt

//-----------------------------------------------------------------------------
// Hauptprogramm main
//-----------------------------------------------------------------------------
void main(void)
{
// Die folgenden 2 Anweisungen stammen vom AVR-Projekt 2-Draht-LCD
  // Setzen des System-Taktes (CPU-Clock),
  // Ausschalten nicht benutzter Hardware-Module
  app_init();                             // Initialisierung der Anwendung
  // Definition von Prototypen fuer LCD-Ausgabe-Funktionen usw.                
  lcd_init();                             // Initialisierung des LCD-Interfaces

  for (n = 0; n < 4; n++)                 // Schreibe 4-zeiligen Display-Text
  {
    lcd_goto_xy(0,n);
    lcd_write_flash_str((fpU08)LCD_STR[n]);
  }

  for (n = 11; n > 0; n--)                // Ermitteln der Anzahl Ziffern
                                          // nach dem Dezimalpunkt
  {
    if(char_wert[n] == '.')               // Punkt gefunden?
    {                                     // JA: hoechster Index vom Array
      i = 11-n;                           // minus n ist die Anzahl Ziffern i 
                                          // nach dem Dezimalpunkt
      if(i > 5) i = 5;                    // i darf nur max. 5 sein
      else;
    }
    else;                                 // NEIN: Punkt suchen
  }
  while (1)                               // Unendliche Schleife
  {
    lcd_goto_xy(8,0);                     // x=8, y=0 (1. Zeile)
    lcd_write_str(char_wert);             // Zahl als String darstellen

    if (float_wert < 0.0)                 // Gleitkomma-Zahl negativ? 
    {                                     // Wenn JA:
      int_wert = -float_wert;             // float negieren und in
                                          // unsigned long int (U32)
                                          // implizit wandeln
      dec_wert = (-float_wert) - int_wert;  // Ziffern nach Punkt ermitteln
      lcd_goto_xy(8,2);                   // x=8, y=2 (3. Zeile)
      lcd_write_float(dec_wert,i,12);     // i Ziffern nach Punkt darstellen
    }
    else                                  // Wenn NEIN (nicht negativ):
    {
      int_wert = float_wert;              // float in U32 implizit wandeln
      dec_wert = float_wert - int_wert;   // Ziffern nach Punkt ermitteln
      lcd_goto_xy(8,2);                   // x=8, y=2 (3. Zeile)
      lcd_write_float(dec_wert,i,12);     // i Ziffern nach Punkt darstellen
    }

    lcd_goto_xy(8,1);                     // x=8, y=1 (2. Zeile)
    lcd_write_U32(int_wert,12);           // Ziffern vor dem Punkt darstellen

    lcd_goto_xy(8,3);                     // x=8, y=3 (4. Zeile)
    lcd_write_float(float_wert,i,12);     // Gleitkomma-Zahl darstellen
  }
}
//--------------------------------------------------------------------------------------------------------------------
